#' @title Vector Autoregressive Model
#' @description Vector autoregressive model. This is a regression method used to
#' establish the temporal relationship between the original time series and
#' the modes generated by quantum walks.
#' @usage qwdap.var(in_data, data_range, verbose, plotting)
#' @param in_data a 'QWMS' object, which includes the target series and the
#' selected modes which can be obtained from modes selection.
#' @param data_range the range of the train samples.
#' @param verbose whether to output the result in the console.
#' @param plotting whether to plot.
#'
#' @return a 'QWMODEL' object which includes the information of regression analysis.
#' @import MTS
#' @export
#'
#' @examples
#' data("traffic.n1")
#' res.var <- qwdap.var(traffic.n1,c(1,500))
#' 
qwdap.var<-function(in_data, data_range, verbose = TRUE,plotting = FALSE){
  if(class(in_data)!='QWMS'){
    print("The 'in_data' is not a 'QWMS' object.")
    return()
  }
  if(!is.vector(data_range)||!is.numeric(data_range)||length(data_range)<2){
    print("The parameter 'data_range' is error.")
    return()
  }
  # pre combine
  co_data = cbind(in_data$real, in_data$ctqw)
  co_data <- subset(co_data, select = c(colnames(in_data$real), in_data$variate))
  indexes<-VARXorder(co_data[data_range[1]:data_range[2],1],
                     co_data[data_range[1]:data_range[2],-1],10)
  if(indexes$aicor[1]==0){
    indexes$aicor[1]=indexes$bicor[1]
  }
  ## 使用VARX
  res<-NA
  if(verbose){
    res <- MTS::VARX(co_data[,1],indexes$aicor[1],co_data[,-1],indexes$aicor[2],output = T)
  }else{
    res <- MTS::VARX(co_data[,1],indexes$aicor[1],co_data[,-1],indexes$aicor[2],output = F)
  }
  if(plotting){
    plot(x=c(1:length(co_data[,1])),y=co_data[,1],type="l",col=1,xlab="X",ylab="Y")
    cir<-c((indexes$aicor[1]+1):length(co_data[,1]))
    lines(x=cir,y=res$residuals[,1]+co_data[,1][cir],type="l",col=2)
  }
  res<-list(real = in_data$real, ctqw = co_data[,-1], index = in_data$index,
            method = "VAR",model=res)
  res<-structure(res,class="QWMODEL")
  return(res)
}
